#!/usr/bin/python
#
# Copyright (C) 2012, Canonical Ltd (http://www.canonical.com/)
#
# This file is part of ubuntu-test-cases.
#
# ubuntu-test-cases is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# ubuntu-test-cases is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ubuntu-test-cases.  If not, see
# <http://www.gnu.org/licenses/>.

"""Ubiquity installation works correctly."""

import logging
import unittest
import ldtp
import ooldtp
import time


class UbiquityTest(unittest.TestCase):
    MAIN_WINDOW = 'frmlive_installer'
    BTN_FORWARD = 'btnnext'
    BTN_INSTALL_NOW = 'btnnext'
    RBTN_USE_ENTIRE_DISK = 'rbtnusedevice'
    RBTN_LOGIN_AUTOMATICALLY = 'rbtnloginauto'

    TXT_LOCATION = 'txttimezonecityentry'  # Timezone selector
    TXT_FULLNAME = 'txtfullname'
    TXT_HOSTNAME = 'txthostname'
    TXT_USERNAME = 'txtusername'
    TXT_PASSWORD1 = 'txtpassword'  # Password
    TXT_PASSWORD2 = 'txtverifiedpassword'  # Password confirmation

    def _waittillenabled(self, ui, component_name, timeout=300):
        """Wait until widget is enabled."""
        ldtp.wait(15)  # UI is slow to react sometimes
        component = ui.getchild(component_name)
        while not component.stateenabled():
            logging.debug('Waiting for {}. Till {} to go'
                          .format(component_name, timeout))
            ldtp.wait(15)
            timeout -= 15
            if timeout < 0:
                return False
        return True

    def test_ubiquity(self):
        """Ubiquity installation works correctly."""
        # Content to fill the forms with
        data = {
            'location': 'London',
            'fullname': 'Ubuntu Smoke Test User',
            'hostname': 'smoketest',
            'username': 'ubuntu',
            'password1': 'ubuntu',
            'password2': 'ubuntu'
        }

        # Give time for LDTP to start up
        logging.info('Sleeping for 5s')
        time.sleep(5)

        logging.info('Launching Ubiquity')
        ldtp.launchapp('ubiquity', ['gtk_ui'])
        ldtp.waittillguiexist(self.MAIN_WINDOW)
        ldtp.wait(30)  # Ubiquity in a VM is sooo sloooow
        ui = ooldtp.context(self.MAIN_WINDOW)

        # First Screen
        logging.info('Welcome Screen')
        ldtp.wait(30)
        ui.click(self.BTN_FORWARD)
        self._waittillenabled(ui, self.BTN_FORWARD)

        # Choose here if you want to install stuff from network
        logging.info('Preparing to install Ubuntu')
        logging.info('Leave defaults')
        ui.click(self.BTN_FORWARD)
        self._waittillenabled(ui, self.BTN_FORWARD)

        # Drive setup
        logging.info('Allocate Disk Drive')
        logging.info('Use entire partition')
        ui.click(self.RBTN_USE_ENTIRE_DISK)
        ui.click(self.BTN_FORWARD)
        ldtp.wait(30)
        ui.remap()
        ldtp.wait(30)
        self._waittillenabled(ui, self.BTN_INSTALL_NOW)

        logging.info('Allocate Drive Space')
        logging.info('Leave defaults')
        ui.click(self.BTN_INSTALL_NOW)
        ui.remap()
        ldtp.wait(30)

        # Location
        logging.info('Where are you')
        ui.settextvalue(self.TXT_LOCATION, data['location'])
        ui.click(self.BTN_FORWARD)
        self._waittillenabled(ui, self.BTN_FORWARD)

        # Keyboard
        logging.info('Keyboard Layout')
        ui.click(self.BTN_FORWARD)
        self._waittillenabled(ui, self.BTN_FORWARD)

        # User Identity
        logging.debug('Who are you?')
        ui.settextvalue(self.TXT_FULLNAME, data['fullname'])
        ui.settextvalue(self.TXT_HOSTNAME, data['hostname'])
        ui.settextvalue(self.TXT_USERNAME, data['username'])
        ui.settextvalue(self.TXT_PASSWORD1, data['password1'])
        ui.settextvalue(self.TXT_PASSWORD2, data['password2'])
        ui.click(self.RBTN_LOGIN_AUTOMATICALLY)
        ldtp.wait(30)
        ui.click(self.BTN_FORWARD)

        ldtp.wait(30)
        ldtp.waittillguiexist('dlgfinished_dialog', guiTimeOut=60 * 20)

        ldtp.wait(30)

        # Don't reboot just verify that ubiquity is done
        self.assertTrue(ldtp.guiexist('dlgfinished_dialog', 'btnrebootbutton'))

        # TODO:
        # Execute here steps to setup pan boot tests
        # Do not reboot here otherwise, the parent script will be interupted
        if ldtp.guiexist('dlgfinished_dialog', 'btnrebootbutton'):
            ldtp.click('dlgfinished_dialog', 'btnquitbutton')

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG,
                        format='%(levelname)s: %(message)s')
    unittest.main()
